﻿package com.incomingfodder 
{
	import com.incomingfodder.sound.ArrowShoot;
	import flash.display.Graphics;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.geom.Point;
	
	/**
	* An arrow
	* @author <christoffer@enedahl.com>
	*/
	public class Arrow extends BaseObject
	{
		var Speed:Number;
		var Power:Number;
		var RangeLeft:Number;
		var Direction:Point;
		
		var deltaX:Number;
		var deltaY:Number;
		
		/**
		 * When arrow hit something, it stays for a while, then disposes
		 */
		var hitTime:Number = 0;
		
		/**
		 * Create a new arrow, send it away in specifyed direction
		 * @param	speed
		 * @param	power
		 * @param	angle
		 * @param	tx
		 * @param	ty
		 * @param	range
		 * @param	direction
		 */
		public function Arrow( speed:Number, power:Number, angle:Number, tx:int, ty:int, range:Number, direction:Point, nosound:Boolean = false ) 
		{
			Tx = tx;
			Ty = ty;
			
			x = Tx * Game.TileSize + (Game.TileSize*.5);
			y = Ty * Game.TileSize + (Game.TileSize * .5);
			
			Speed = speed;
			Power = power;
			RangeLeft = range;
			Direction = direction;
			
			this.rotation = angle * (180 / Math.PI);
			
			this.addEventListener( Event.ENTER_FRAME, onEnterFrame, false, 0, true);
			
			if(!nosound ){
				var a:ArrowShoot = new ArrowShoot();
				a.play();
			}
		}
		
		/**
		 * Remove, cleanup events
		 */
		public override function dispose()
		{
			this.removeEventListener( Event.ENTER_FRAME, onEnterFrame, false);
			if(this.parent!=null){
				this.parent.removeChild(this);
			}
		}
		
		/**
		 * Move arrow
		 * @param	e
		 */
		public function onEnterFrame(e:Event) 
		{			
			if ( !Game.IsRunning ) return; //Not running;
			x += Direction.x * Speed;
			y += Direction.y * Speed;
			
			if (hitTime != 0)
			{
				if (hitTime + 130 < new Date().time)
				{
					dispose();
				}
				return;
			}
			
			RangeLeft -= Direction.length*Speed;
			
			if (RangeLeft <= 0)
			{			
				this.dispose();
				return;
			}
			
			var newTx:int = Math.floor( x / Game.TileSize);
			var newTy:int = Math.floor( y / Game.TileSize);
			
			tryHit(newTx, newTy);
			
		}
		
		/**
		 * Try hit as location, override this for hitting other stuff than monsters
		 * @param	newTx
		 * @param	newTy
		 */
		public function tryHit(newTx:int, newTy:int)
		{
			try{
				var objects:Array = Game.GridMonsters[newTx][newTy] as Array;
				
				if (objects != null && objects.length > 0)
				{
					//TODO choose the closest monster wich is hit (inside bounding circle not tile)
					var monster:Monster = objects[0] as Monster;
					if(monster!= null){
						monster.hit(this);						
						hitTime = new Date().time;
					}
					
				}
			}catch(e){}
		}
		
	}
	
}