﻿package com.incomingfodder 
{
	import com.incomingfodder.sound.Coin;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.media.SoundTransform;
	import flash.utils.Timer;
	
	/**
	* The core game object
	* @author <christoffer@enedahl.com>
	*/
	public class Game 
	{
		static public var TileSize:uint = 32;
		static public var Grid:Array; // [x][y] TileObjects
		static public var GridMap:Map;
		static public var GridWidth:uint = 20;
		static public var GridHeight:uint = 15;
		static public var Monsters:Array;
		
		static public var Timers:Array; //Timers to clear on dispose
		
		static public var MonsterGoals:Array; //Castle, mines etc.. not towers
		
		static public var IsRunning:Boolean;
		
		/**
		 * Reference to monsters for quick lookup
		 */
		static public var GridMonsters:Array;
		
		/**
		 * Canvas to add sprites to
		 */
		static public var Canvas:MovieClip;
		
		/**
		 * What resources we got
		 */
		static public var ResourceTree:int;
		static public var ResourceGold:int;		
		
		static public var Level:uint;
		
		
		static private var levels:Array = [
			{type: AStarMonster, level: 1, count: 5, x:0, y:7 },
			{type: Monster, level: 1, count: 5, x:0, y:7 },
			{type: Cannon, level: 1, count: 5, x:0, y:7 },
			
			{type: AStarMonster, level: 2, count: 5, x:10, y:0 },
			{type: Monster, level: 2, count: 5, x:10, y:0 },
			{type: Cannon, level: 2, count: 5, x:10, y:0 },
			
			{type: AStarMonster, level: 3, count: 5, x:0, y:7 },
			{type: Monster, level: 3, count: 5, x:0, y:7 },
			{type: Cannon, level: 3, count: 5, x:0, y:7 },			
			
			{type: AStarMonster, level: 4, count: 5, x:17, y:0 },
			{type: Monster, level: 4, count: 5, x:17, y:0 },
			{type: Cannon, level: 4, count: 5, x:17, y:0 },			
			
			{type: AStarMonster, level: 5, count: 5, x:0, y:7 },
			{type: Monster, level: 5, count: 5, x:0, y:7 },
			{type: Cannon, level: 5, count: 5, x:0, y:7 },			
			
			{type: AStarMonster, level: 6, count: 5, x:0, y:0 },
			{type: Monster, level: 6, count: 5, x:0, y:0 },
			{type: Cannon, level: 6, count: 5, x:0, y:0 },			
			
			{type: AStarMonster, level: 7, count: 5, x:0, y:14 },
			{type: Monster, level: 7, count: 5, x:0, y:14 },
			{type: Cannon, level: 7, count: 5, x:0, y:14 },
			
			];
			
		public function Game( level:uint, document:Document  ) 
		{
			Level = level;
			
			ResourceGold = 10;
			ResourceTree = 10;
			IsRunning = true;
			
			//Create canvas and hook up mouse
			Canvas = new MovieClip();
			GuiMap.placeholder.addChild( Canvas );
			//document.addChild( Canvas );
			document.addEventListener( MouseEvent.MOUSE_DOWN, onClick, false, 0 , true);
			document.addEventListener( MouseEvent.MOUSE_MOVE, onMouseMove, false, 0 , true);
			
			//Init empty grid
			Grid = new Array();	
			GridMonsters = new Array();
			for (var x:uint = 0; x < GridWidth; x++)
			{
				Grid.push( new Array( GridHeight ) );
				GridMonsters.push( new Array( GridHeight ) );
			}
			
			//For aStar
			GridMap = new Map( Grid );
			
			MonsterGoals = new Array();					
			
			//Add castle			
			var cx = Math.floor(GridWidth / 2)-1;
			var cy = Math.floor(GridHeight / 2)-1;
			
			var c:Castle = new Castle(cx, cy);
			Canvas.addChild(c);
			
			MonsterGoals.push( c ); //Add castle to monster goals
			
			
			
			Grid[cx  ][cy  ] = c;
			Grid[cx  ][cy+1] = c;
			Grid[cx+1][cy  ] = c;
			Grid[cx + 1][cy + 1] = c;			
			
			Monsters = new Array();				
			Timers = new Array();			
			
			GuiMap.pause.visible = false;
			GuiMap.defeat.visible = false;
			GuiMap.btn_NewGame.visible = false;
			//GuiMap.pause.addEventListener( MouseEvent.CLICK , onClickPause , false, 0, true);
			
			nextLevel(null); //Launch first level
			
			//Hehe kill the radio to stop the horrible music!
			var r:Radio = new Radio( 18, 1);
			addMonster(r);			
			
		}
		
		
		public function nextLevel(e:TimerEvent)
		{
			trace("nextLevel");
			var waitSeconds = 0;
			if ( e != null)
			{
				removeTimer(e);
				Level++;
				waitSeconds = 5;
			}		
			
			if ( Level >= levels.length)
			{
				return; //No more enemies to send. TODO (Check victory elsewhere?)
			}			
			
			updateHud();
			
			var t:Timer = new Timer(waitSeconds * 1000, 1);
			t.addEventListener( TimerEvent.TIMER, initNextLevel, false, 0, true);
			t.start();
			Timers.push(t);
		}
			
		public function initNextLevel(e:TimerEvent)
		{			
			trace("initNextLevel");
			if ( e != null)
			{
				removeTimer(e);
			}	
			
			var t:Timer = new Timer(2000, levels[Level].count);
			t.addEventListener( TimerEvent.TIMER, spawnMonster, false, 0, true);
			t.addEventListener( TimerEvent.TIMER_COMPLETE, nextLevel, false, 0, true);
			t.start();
			Timers.push(t);
			
		}
		
		
		/**
		 * Clicked that canvas
		 * @param	e
		 */
		public function onClickPause(e:MouseEvent)
		{
			if (IsRunning)
			{
				pause();
			}else
			{
				resume();
			}
		}
		
		/**
		 * Pause the game
		 */
		static public function pause()
		{
			trace("Pause");
			IsRunning = false;
			for (var i:int = 0; i < Timers.length; i++)
			{
				(Timers[i] as Timer).stop();	
			}
			
			Canvas.dispatchEvent( new Event("pauseGame") );
		}
		
		/**
		 * Resume the game after pause
		 */
		static public function resume()
		{
			trace("Resume");
			IsRunning = true;
			for (var i:int = 0; i < Timers.length; i++)
			{
				(Timers[i] as Timer).start();	
			}
			
			Canvas.dispatchEvent( new Event("resumeGame") );
		}
		
		/**
		 * Remove a timer from timers
		 * @param	e
		 */
		public function removeTimer( e:TimerEvent)
		{
			var i:int = Timers.indexOf( e.currentTarget );
		
			if (i >= 0)
			{
				(Timers[i] as Timer).stop(); //TODO removeEventListeners, not bothering now, to little time
				Timers.splice(i, 1);
			}
			
		}
		
		public function spawnMonster(e:Event)
		{
			trace("Trying to spawn " + levels[Level].type);
			
			var m:Monster = new levels[Level].type(levels[Level].x, levels[Level].y);
			
			m.setLevel( levels[Level].level );
			
			addMonster( m );
		}
		
		/**
		 * Adds monster to Canvas, Monsters and GridMonsters
		 * @param	monster
		 */
		static public function addMonster(monster:Monster)
		{
			Canvas.addChild( monster );
			Monsters.push( monster );
			addMonsterInGrid( monster.Tx, monster.Ty, monster );
		}
		
		/**
		 * removes monster to Canvas, Monsters and GridMonsters
		 * @param	monster
		 */
		static public function removeMonster(monster:Monster)
		{
			Canvas.removeChild( monster );
			try{
				Monsters.splice( Monsters.indexOf( monster ), 1);
			}catch(e){}
			removeMonsterInGrid( monster );
			
			if (Monsters.length <= 1)
			{
				if (Level >= levels.length)
				{
					
					if (Monsters.length == 1)
					{//Check if monster is radio, then ok, victory
					    try
					    {
						   var r:Radio = Monsters[0] as Radio;
						}catch (e:Error)
						{
							return; //Not radio
						}	
					}
					
					GuiMap.victory.visible = true;
					trace("Victory!");
				}
			}
		}
		
		
		/**
		 * Move a monster from one tile to another
		 * @param	FromX
		 * @param	FromY
		 * @param	monster
		 */
		static public function moveMonsterInGrid( ToX:int, ToY:int, monster:Monster)
		{
			//remove
			removeMonsterInGrid(monster);
			
			//Add			
			addMonsterInGrid(ToX, ToY,monster);			
		}
		
		/**
		 * Remove monster from grid
		 * @param	FromX
		 * @param	FromY
		 * @param	monster
		 */
		static public function removeMonsterInGrid(monster:Monster)
		{
			//remove
			try
			{
				var objects:Array = GridMonsters[monster.Tx][monster.Ty] as Array;
				var index:int = objects.indexOf( monster );
				objects.splice( index, 1); //Remove from list
			}catch (e:Error)
			{ }
		}
		
		/**
		 * add monster to grid
		 * @param	monster
		 */
		static public function addMonsterInGrid(ToX:int, ToY:int, monster:Monster)
		{
			try{
				var toobjects:Array = GridMonsters[ToX][ToY] as Array;			
				if (toobjects == null)
				{
					toobjects = new Array();
					GridMonsters[ToX][ToY] = toobjects;
				}			
				toobjects.push( monster);	
				
			}catch (e:Error){}
		}
		
		/**
		 * Add gold to the resource
		 * @param	amount
		 */
		static public function addGold(amount:int)
		{
			ResourceGold += amount;
			updateHud();
		}
		
		/**
		 * Update gold etc in hud
		 */
		static public function updateHud()
		{
			var lev:Number = Level + 1;
			
			GuiMap.coinAmount.text = ResourceGold.toString() + "   Level "+ lev.toString();			
		}
		
		/**
		 * Monsert hit tower, castle, mine at location,
		 * @param	tx
		 * @param	ty
		 * @param	monster
		 */
		static public function hit(tx, ty, monster:Monster)
		{		
			var target:BaseObject;
			
			try{
				target = Grid[tx][ty] as BaseObject;
			}catch (e)
			{ }
			
			if (target != null)
			{
				target.Hit( monster.Power );				
			}			
		}
		
		/**
		 * Removes an object in the grid
		 * @param	tx
		 * @param	ty
		 */
		static public function removeObject( tx:uint ,ty:uint ) 
		{
			try
			{
				try{
					var obj:BaseObject = Grid[tx][ty] as BaseObject;
					//Canvas.removeChild( obj );
					if(obj!= null){
						obj.dispose();
					}
					
				}catch (e:Error)
				{
					trace("Game.removeObject: " + e.message);
				}
				
				Grid[tx][ty] = null;
			}catch(e){}
		}
		
		/**
		 * If a tile is empty/free
		 * @param	x tile x
		 * @param	y tile y
		 */
		static public function isTileFree(x, y)
		{
			try{
				return (Grid[x][y] == null);
			}catch (e)
			{
				return false;
			}			
		}
		
		//Will substract the cost if it can afford it, if not returns false;
		static public function substractResource(cost:ResourceCost):Boolean
		{
			if ( cost.Gold <= ResourceGold && cost.Trees <= ResourceTree)
			{
				ResourceGold -= cost.Gold;
				ResourceTree -= cost.Trees;
				
				updateHud();
				return true;
			}
			return false;			
			
		}
		
		static public function onCastleDestroyed() 
		{
			
			pause();
			
			GuiMap.defeat.visible = true;
			//GuiMap.btn_NewGame.visible = true;			
			
			Canvas.parent.removeEventListener( MouseEvent.MOUSE_DOWN, onClick, false);
			Canvas.parent.removeEventListener( MouseEvent.MOUSE_MOVE, onMouseMove, false);
			
		    dispose();			
			
		}
		
		/**
		 * Cleanup
		 */
		static public function dispose()
		{
			if(Monsters != null){
				for each(var m:Monster in Monsters)
				{
					m.dispose();
				}
				Monsters = null;
			}
			GridMonsters = null;			
			
			if(Grid != null){
				for (var x:uint = 0; x < GridWidth; x++)
				{
					for (var y:uint = 0; y < GridHeight; y++)
					{
						var b:BaseObject = Grid[x][y] as BaseObject;
						try{
							Canvas.removeChild(b);
						}catch(e){}
						b.dispose();
						
						Grid[x][y]  = null;
					}
					Grid[x] = null;
				}
				Grid = null;
			}
			
		}
		
		/**
		 * Clicked that canvas
		 * @param	e
		 */
		static public function onClick(e:MouseEvent)
		{		
			var tx = Math.floor( e.stageX / Game.TileSize );
			var ty = Math.floor( e.stageY / Game.TileSize );
			
			
			//if (e.shiftKey && Game.isTileFree( tx, ty))
			//{
				//addMonster( new Cannon(tx, ty) );
				//
				//return;
			//}
			
			if ( (e.ctrlKey||e.shiftKey) && Game.isTileFree( tx, ty))
			{
				var c:Casino = new Casino( tx, ty );			
				if ( !c.IsValid ) return; //Could not create casino
				
				Canvas.addChild( c );
			
				return;
			}
			
			var t:Tower = new Tower( tx, ty );			
			if ( !t.IsValid ) return; //Could not create tower
			
			Canvas.addChild( t );
			
		}
		
		/**
		 * Moved the mouse over the canvas
		 * @param	e
		 */
		static public function onMouseMove( e:MouseEvent)
		{
		
			var tx = Math.floor( e.stageX / Game.TileSize );
			var ty = Math.floor( e.stageY / Game.TileSize );
			
			if( tx >0 && tx < GridWidth && ty >0 && ty < GridHeight){
				GuiMap.selection.visible = true;
				GuiMap.selection.x = tx * Game.TileSize;
				GuiMap.selection.y = ty * Game.TileSize;
			}else
			{
				GuiMap.selection.visible = false;				
			}
			
		}
	}
	
}