﻿package com.incomingfodder 
{
	import com.incomingfodder.sound.TowerBuild;
	import com.incomingfodder.sound.TowerComplete;
	import com.incomingfodder.sound.TowerDenied;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.geom.Point;
	import flash.media.SoundChannel;
	import flash.utils.Timer;
	
	/**
	* Base tower
	* @author <christoffer@enedahl.com>
	*/
	public class Tower extends BaseObject
	{
		/**
		 * Build costs
		 */
		public var Cost:ResourceCost = new ResourceCost(0, 2, 3000);
		
		/**
		 * How far can this tower shoot
		 */
		public var Range: int = 3;
		/**
		 * How fast
		 */
		public var Speed: Number = 5;
		/**
		 * How powerful impact
		 */
		public var Power: Number = 0.5; //0 for debug, do not inflict damage
		
		/**
		 * MS how long
		 */
		private var reloadTime:Number = 1500;
		
		
		/**
		 * When was last attack
		 * */
		private var lastAttack:Number = 0;
		
		
		/**
		 * If the tower is created on a valid spot
		 */
		public var IsValid:Boolean = false;
		
		/**
		 * How many MS left before complete
		 */
		public var BuildTimeLeft:Number;
		
		public var IsBuilt:Boolean = false;		
		/**
		 * Build sound, stop when don building
		 */
		private var sndChnBuild:SoundChannel ;
		
		/**
		 * 
		 * @param	tx tile x location
		 * @param	ty tile y location
		 */
		public function Tower( tx:uint , ty:uint, alreadyBuilt:Boolean = false ) 
		{
			
			IsValid = true;
			
			Tx = tx;
			Ty = ty;
			
			if(!alreadyBuilt){
				if ( !Game.isTileFree( Tx, Ty) ) //Free spot
				{
					IsValid = false;
				}else
				{
					
					if (Tx == 0 || Ty == 0 || Tx == Game.GridWidth - 1 || Ty == Game.GridHeight - 1 )
					{
						IsValid = false; //Do not allow build on outer most tiles
					}else{
					
						if ( !Game.substractResource( Cost )){ //Money
							IsValid = false;
						}else
						{
							//Add the tower
							try{
								Game.Grid[Tx][Ty] = this;
							}catch (e:Error){
								IsValid = false;
							}
						}
					}
				}
			}
			
			if (!IsValid)
			{
				//Play sound, cannot place tower here
				var d:TowerDenied = new TowerDenied();
				d.play();
				return;
			}
			
			//Init tower
			
			addEventListener( Event.ENTER_FRAME, onEnterFrame, false, 0, true);
			
			Health = 10;
			Strength = 10;
			
			this.x = Tx * Game.TileSize;
			this.y = Ty * Game.TileSize;
			
			if (alreadyBuilt)
			{
				BuildTimeLeft = 0;
				IsBuilt = true;
			}else{
				BuildTimeLeft = Cost.Time + new Date().time;
				
				var b:TowerBuild = new TowerBuild();
				sndChnBuild = b.play();
			}	
			
		}
		
		public override function dispose()
		{			
			removeEventListener( Event.ENTER_FRAME, onEnterFrame, false);
			
			if (this.parent != null)
			{
				this.parent.removeChild( this);
			}
			
			if (sndChnBuild != null)
			{
				sndChnBuild.stop();
				sndChnBuild = null;
			}
		}
		
		public function onEnterFrame(e:Event)
		{
		
			if ( !Game.IsRunning ) return; //Not running;
			
			if (IsBuilt)
			{
				var now:Number = new Date().time;
				
				//If reloadtime is done
				if( (lastAttack + reloadTime) < now){
									
					//Search for target
					var monster:Monster = getBestTarget();
					
					//Fire
					if (monster != null)
					{
						attack(monster);
						lastAttack = now;						
					}
				
				}
				
			}else
			{
				var timeLeft:Number = new Date().time - BuildTimeLeft
				if (timeLeft > 0)
				{
					IsBuilt = true;
					gotoAndStop( this.totalFrames); //Building is complete, jump to last frame
					if(sndChnBuild!=null){
						sndChnBuild.stop();
					}
					
					var t:TowerComplete = new TowerComplete();
					t.play();
					
				}else
				{
					//Animate building
					var percent = 1 + ( timeLeft / Cost.Time);
					
					gotoAndStop( Math.floor(totalFrames * percent));
				}
			}
			
		}
		
		private function attack(monster:Monster)
		{
			var deltaX:int = monster.x - (Tx*Game.TileSize + (Game.TileSize *.5));
			var deltaY:int = monster.y - (Ty*Game.TileSize + (Game.TileSize *.5));
			
			var p:Point = new Point(deltaX, deltaY);
			p.normalize(1);
			if (p.length == 0)
			{
				trace("Target too close");
				return;
			}
			
			var angle:Number = Math.atan2( deltaY, deltaX);
			
			var a:Arrow = new Arrow( Speed, Power, angle, Tx, Ty, Range * Game.TileSize, p);
			
			Game.Canvas.addChild( a );
			
		}
		
		private function getBestTarget():Monster
		{
			var ClosestDist:Number = Number.MAX_VALUE;
			var ClosestObject:Monster;
			
			for	each( var obj:BaseObject in Game.Monsters)
			{
				var distX = obj.Tx - Tx;
				if (Math.abs(distX) > Range) continue;
				
				var distY = obj.Ty - Ty;
				if (Math.abs(distY) > Range) continue;
				
				var dist2 = (distX * distX) + (distY * distY);
				var range2 = Range * Range;
				if (dist2 < range2)
				{
					if (ClosestDist < dist2) continue;
					
					ClosestDist = dist2;
					ClosestObject = obj as Monster;
				}
			}
			
			return ClosestObject;
		}
		
	}
	
}